#!/usr/bin/tixwish4.1.8.3
#####################################################################
# Copyright (C) 2000 Artur Trzewik
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
######################################################################
# Header
# CVS - $Id: FormServer.tcl,v 1.17 2001/01/06 14:50:04 artur Exp $
#
# Information:
#
# Dies ist eine Startdatei fr ein FormularServer
# Das Programm liest die xmlform Datei (Beschreibung der DB Formulare als XML)
# verbindet sich mit Datenbank
# und setz die Operationen auf Formularen in SQL Anweisungen

# Finde Program-Verzeichnis
set sname [info script]
if {$sname==""} {
    # Run interactive for develop purposes
    set progdir /home/artur/diplom/program
} else {
    file lstat $sname stats
    # follow sym links
    if {$stats(type)=="link"} {
	set sname [file readlink $sname]
	if {[file pathtype $sname]=="relative"} {
	    set sname [file join [file dirname [info script]] $sname]
	}
    }
    set progdir [file dirname $sname]
}
wm withdraw .
# Lade Weitere Tcl Komponenten Xotc,tDom,TclX
source [file join $progdir Init.tcl]

#Lade andere Module
source [file join $progdir sql.tcl]
source [file join $progdir formengine.tcl]
source [file join $progdir formfactory.tcl]
source [file join $progdir dbproxy.tcl]
#source [file join $progdir tableview.tcl]
source [file join $progdir fe_services.tcl]

#Dom Schnittstelle auch auf deutsche Umlauten setzten
dom setResultEncoding iso8859-1
#Setzte Array lan mit bersetzung-Strings
checklanguage fs

# Haupt-Objekt des Programm das die GUI aufbaut und kontroliert
# Nur ein per Ausfhrung (singleton)

Object FormServer
FormServer proc setwin win {
    [self] instvar window opts
    set window $win
    global lan
    menu $win.mb -tearoff 0
    menu $win.mb.file -tearoff 0
    menu $win.mb.view -tearoff 0
    menu $win.mb.help -tearoff 0
    
    $win.mb add cascade -label $lan(menu.file.file) -menu $win.mb.file \
	-underline 0
    $win.mb add cascade -label $lan(menu.help.help) -menu $win.mb.help \
	-underline 0
    
    $win.mb.file add command -label $lan(menu.file.readxmlforms) \
	-command {FormServer readxmlforms}
    $win.mb.file add command -label $lan(menu.file.saveforms) \
	-command {FormServer saveForms}
    $win.mb.file add command -label $lan(menu.file.saveformsas) \
	-command {FormServer saveFormsAs}
    $win.mb.file add command -label $lan(menu.file.loadmacros) \
	-command {FormServer loadMacros}
    $win.mb.file add separator
    $win.mb.file add command -label $lan(menu.file.dbconnect) \
	-command {FormServer dbconnect}
    $win.mb.file add command -label $lan(menu.file.dbdisconnect) \
	-command {FormServer dbdisconnect}
    $win.mb.file add separator
    $win.mb.file add command -label $lan(menu.file.exit) \
	-command {FormServer handle_exit}

    $win.mb.help add command -label $lan(menu.help.help) \
	-command handle_help
    $win.mb.help add command -label $lan(menu.help.about) \
	-command {handle_about FormServer}
    $win.mb.help add command -label $lan(menu.help.homepage) \
	-command wwwlink_autor
    if {$opts(debug)} {
	$win.mb.help add separator
	$win.mb.help add command -label $lan(menu.help.debug) \
	    -command {commandloop}
	$win.mb.help add command -label inspector \
	    -command {source [file join $progdir inspector.tcl]}
	$win.mb.help add command -label win_inspector \
	    -command {source [file join $progdir win_inspector.tcl]}
	$win.mb.help add command -label "view XML Forms" \
	    -command {
		if {![winfo exist .xmlview]} {
		    toplevel .xmlview
		    tixScrolledText .xmlview.st
		    pack  .xmlview.st -expand yes -fill both
		}
		.xmlview.st subwidget text delete 1.0 end
		.xmlview.st subwidget text insert 1.0 [$FormServer::data_node asXML]
	    }
	$win.mb.help add command -label "sql DEBUG" \
	    -command {source [file join $progdir sql_debug.tcl]}
    }
    tixScrolledTList $win.list  -options {
	tlist.orient horizontal
	tlist.selectMode single
	tlist.command {FormServer executeForm}
    }
    pack $win.list -expand yes -fill both 
    
    if {$win==""} {
	. configure -menu $win.mb
	wm protocol . WM_DELETE_WINDOW "[self] handle_exit"
    } else {
	$win configure -menu $win.mb
	wm protocol $win WM_DELETE_WINDOW "[self] handle_exit"
    }
}
# bei Erfolg return value ist wahr
# vorsicht die Funktion wird in zwie fllen benutzt
# 1. Bei Programmstart und automatisch die xmlforms zu laden, die als
#    Programmparameter eingegeben wurden $ffile!=""
# 2. Als callback von Men loadforms
FormServer proc readxmlforms {{ffile {}}} {
    [self] instvar window data_node dom_doc db_name sqlconnect form_nodelist \
	file opts file_name
    global lan
    set file $ffile
    # !!! Suche andere Methode das Tree window zu finden
    set types {
	{"XML Forms"		{.xmlforms}	}
	{"All files"		*}
    }
    if {$file==""} {
	set file [tk_getOpenFile -filetypes $types -parent .]
	if {$file == ""} { return 0}
    } else {
	if {![file exist $file] || ![file isfile $file]} {
	    tk_messageBox -message [format $lan(message.nofile) $file] -icon error -type ok
	    return 0
	}
    }
    if {[info exist sdom_doc]} {
	set alt_dom_doc $dom_doc
    } else {
	set alt_dom_doc {}
    }
    set flist [$window.list subwidget tlist]
    set fileh [open $file]
    set dom_doc [dom parse [read $fileh]]
    close $fileh
    set data_node [$dom_doc documentElement]
    if {[$data_node nodeName]!="dbforms"} {
	$dom_doc delete
	set dom_doc ""
	tk_messageBox -message $lan(message.wrongxml) -icon error -type ok
	return 0
    }
    set file_name $file
    set db_name [$data_node getAttribute name]
    set dbconnect_par {}
    set form_nodelist {}
    $flist delete 0 end
    foreach node [$data_node childNodes] {
	if {[$node nodeType]=="ELEMENT_NODE" && [$node nodeName]=="form"} then {
	    $flist insert end -itemtype imagetext \
		-text [$node getAttribute name] \
		-image [tix getimage openfold]
	    lappend form_nodelist $node
	} elseif {[$node nodeType]=="PROCESSING_INSTRUCTION_NODE" && [$node target]=="dbconnect"} {
	    set dbconnect_par [$node data]
	} elseif {[$node nodeType]=="PROCESSING_INSTRUCTION_NODE" && [$node target]=="macros"} {
	    if {!($ffile!="" && ($opts(ignorexmlpar) || $opts(loadmacros)!=""))} {
		if {[tk_messageBox -message [format $lan(dialog.loadmacros)  [$node data]] -icon question -type okcancel]=="ok"} {
		    [self] loadMacroFile [$node data]
		} 
	    }
	}
    }
    # Alte Strukturen und SQL-Verbindung falls vorhanden Lschen
    [self] dbdisconnect
    wm title . [file tail $file]:$db_name
    # berschreibe die xmlcodierte parameter mit program-option parameter
    # wenn -xmlignore par dann xml parameter ganz lschen
    if {$ffile!=""} {
	if {$opts(ignorexmlpar)} {
	    set dbconnect_par {}
	}
	if {$opts(connect)!=""} {
	    set opts_connect $opts(connect)
	    foreach con [keylkeys opts_connect] {
		keylset dbconnect_par $con [keylget opts_connect $con]
	    }
	}
	if {$opts(noconnectdialog)} {
	    set sqlconnect [ConnectCreator getconnect [Object autoname sqlconnect] $dbconnect_par 0 1]
	} else {
	set sqlconnect [ConnectCreator getconnect [Object autoname sqlconnect] $dbconnect_par]
	}
    } else {
	set sqlconnect [ConnectCreator getconnect [Object autoname sqlconnect] $dbconnect_par]
    }
    if {$sqlconnect==""} {
	tk_messageBox -message $lan(connect.noconnect) -type ok -icon warning
    }
    if {$alt_dom_doc!=""} {
	$alt_dom_doc delete
    }
    return 1
}
FormServer proc saveForms {} {
    [self] instvar file_name
    if {![info exists file_name]} {
	[self] saveFormsAs
    } else {
	[self] saveFormsAs $file_name
    }
}
FormServer proc saveFormsAs {{f_name {}}} {
   [self] instvar data_node file_name window db_name
    set types {
        {"XML files"         {.xmlforms}  }
        {"All files"            *}
    }
    if {$f_name!=""} {
	set file_name $f_name
    } else {
	if {![info exists file_name]} {
	    set file_name $db_name.xmlforms
	}
	set f_name [tk_getSaveFile -filetypes $types -parent . \
            -initialfile $file_name -defaultextension .xmlforms]
	if {$f_name==""} return
    }
    set file_name $f_name
    set fh [open $file_name w]
    puts $fh "<?xml version='1.0' encoding='iso-8859-1'?>\n<!DOCTYPE dbforms SYSTEM \"xmlforms.dtd\">\n[$data_node asXML]\n"
    close $fh
    [self] needSave 0
}
FormServer proc needSave {{bool 1}} {
    [self] instvar need_save
    set need_save $bool
}
FormServer proc loadMacroFile mfile {
    if {![file exist $mfile]} {
	[self] instvar file
	if {[file exist [file join [file dirname $file] $mfile]]} {
	    source [file join [file dirname $file] $mfile]
	} else {
	    global lan
	    tk_messageBox  -message [format $lan(dialog.nomacrofile) $mfile] -type ok -icon error
	}
    } else {
	source $mfile
    }
}
FormServer proc loadMacros {} {
    set types {
	{"tcl Files"		{.tcl}	}
	{"All files"		*}
    }
    set file [tk_getOpenFile -filetypes $types -parent .]
    if {$file == ""} { return }
    [self] loadMacroFile $file
}
FormServer proc executeForm num {
    [self] instvar window data_node form_nodelist sqlconnect opts  
    #set flist [$window.list subwidget tlist]
    #set name [$flist entrycget $num -text]
    #Work Arround find bringt do Maschine zum Absturz
    #set form_node [$data_node find name $name]
    set fname [Object autoname myFormEngine]
    set form_node [lindex $form_nodelist $num]
    set form_name [$form_node getAttribute name]
    FormEngine createPerName $form_name $fname -form_node \
	$form_node -sqlconnect $sqlconnect 
    $fname map
    $fname reload
}
FormServer proc dbconnect {} {
    [self] instvar sqlconnect
    global lan
    if {[info exist sqlconnect] && $sqlconnect!=""} {
	tk_messageBox -message $lan(dialog.isconnect) -type ok -icon info
	return
    }
    set sqlconnect [ConnectCreator getconnect [Object autoname sqlconnect] ""]
    if {$sqlconnect==""} {
	tk_messageBox -message $lan(connect.noconnect) -type ok -icon warning
    }

}
FormServer proc dbdisconnect {} {
    [self] instvar sqlconnect
    if {[info exist sqlconnect] && $sqlconnect!=""} {
	$sqlconnect destroy
	set sqlconnect ""
    }
}
# Probiert die Optionen unter opts auszufhren
FormServer proc handleOpts {} {
    [self] instvar opts sqlconnect data_node form_nodelist
    [self] setwin {}
    if {$opts(xmlfile)!=""} {
	if {[[self] readxmlforms $opts(xmlfile)]} {
	    if {$opts(loadmacros)!=""} {
		loadMacroFile $opts(loadmacros)
	    }
	    if {$sqlconnect!="" && $opts(openform)!=""} {
		set node [$data_node selectNodes child::form\[@name=\"$opts(openform)\"\]]
		if {$node!=""} {
		    [self] executeForm [lsearch $form_nodelist $node]
		} else {
		    global lan
		    tk_messageBox -message [format $lan(message.noform) $opts(openform)] -icon error -type ok
		}
	    }
	}
    }
    wm deiconify .
}
FormServer proc handle_exit {} {
    [self] instvar need_save
    global lan
    if {[tk_messageBox -message $lan(dialog.endmsg) -icon question -type okcancel]=="ok"} { 
	if {$need_save} {
	    if {[tk_messageBox -message $lan(dialog.saveforms) -icon question -type okcancel]=="ok"} {
		[self] saveFormsAs
	    }
	}
	exit 
    }
}
# FormServer Initialiesierung
FormServer set need_save 0

# default Optionen fr FormServer Setzen
array set FormServer::opts {
    xmlfile {} loadmacros {} viewonly 0 openform {} connect {} debug 0 
    noconnectdialog 0  ignorexmlpar 0
}
#Die FormEngine mssen zuerst zerstr werden weil sie 
#beim delete noch die SQL-Connect brauchen
set globalexit 0
Object setExitHandler {
    global globalexit
    set globalexit 1
#     foreach obj [FormEngine info instances] {
# 	if {[FormEngine info instances $obj]!=""} {
# 	    puts "destroying $obj"
# 	    $obj destroy
# 	}
#     }
}

#Auswertung von Programparameter
if {!$tcl_interactive && [llength $argv]>=1} {
    if {[llength $argv]==1 && $argv!="-help" && ![string match -* $argv]} {
	set FormServer::opts(xmlfile) $argv
    } else {
	package require opt
	set pkey [::tcl::OptProc ::tcl::OptParseFS {
	    {-connect -string {} "db connect parameters f.e. {interface mysql} {dbname dbank} {user root}"}
	    {-noconnectdialog "connect to database without connect dialog"}
	    {-viewonly "use forms only for viewing data"}
	    {-ignorexmlpar "not use xml coded connect parameters"}
	    {-loadmacros -string {} "load macros script"}
	    {-debug "Anable debug functions"}
	    {-openform -string {} "open form by starting application"}
	    {?xmlfile? -string {} "XML formular description file .xmlforms"}
	} {
            foreach v [info locals] {
		set FormServer::opts($v) [set $v]
	    } } ]
	eval ::tcl::OptParseFS $argv
	::tcl::OptKeyDelete $pkey
    }
}
FormServer handleOpts

#toplevel .tab
#TableInterface tabint
#tabint create .tab.tb [$FormServer::data_node select child::tableview] 20
#pack .tab.tb
