#####################################################################
# Copyright (C) 2000 Artur Trzewik
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
######################################################################
# Header
# CVS - $Id: container.tcl,v 1.5 2000/10/15 17:19:20 artur Exp $
#
# Information:
# Modul (benutzt von widgetlib.tcl)
# Eine Biblothek mit Container und Iteratoren in XOTcl
# DoubleList ist entworfen an die Anlehnung an c++ STL
# Allein wegen der Listen hat allerdings den zweifallhaften Sinn
# weil Tcl eigene Listen hat.
# Vielmehr wird hier aus der Benutzung der Iteratoren, Visitors 
# Classen (foreach_elem) oder Compositemglichkeiten gebraucht gemacht.
# Restriktionen: Ein Element kann nur einem Container angehren
Class DLIterator -parameter {
    node
}
DLIterator instproc get {} {
    [self] instvar node
    return $node
}
# Entsprich i++ bei STL C++
DLIterator instproc next_elem {} {
    [self] instvar node
    if {$node=="NIL"} {
	return NIL
    } else {
	set ret $node
	set node [$node set _ln_next]
	return $ret
    }
}
# Entsprinch i-- bei STL C++
DLIterator instproc prev_elem {} {
    [self] instvar node
    if {$node=="NIL"} {
	return NIL
    } else {
	set ret $node
	set node [set ${node}::_ln_prev]
	return $ret
    }    
}

Class DoubleList -parameter {
    {_head NIL}
    {_tail NIL}
}
# Die Elemente des Containers mssen in ihm existieren
# also fr Container db muss das Element z.B. db::element heien

DoubleList instproc push_back eclass {
    [self] instvar _tail _head
    if {[[self] empty]} {
	$eclass set _ln_next NIL
	$eclass set _ln_prev NIL
	set _head $eclass
	set _tail $eclass    
    } else {
	$eclass set _ln_next NIL
	$eclass set _ln_prev $_tail
	$_tail set _ln_next $eclass
	set _tail $eclass
    }
}
DoubleList instproc push_head eclass {
    [self] instvar _tail _head
    if {[[self] empty]} {
	$eclass set _ln_next NIL
	$eclass set _ln_prev NIL
	set _tail $eclass
	set _head $eclass
    } else {
	$eclass set _ln_prev NIL
	$eclass set _ln_next $_head
	$_head set _ln_prev $eclass
	set _head $eclass
    }
}
DoubleList instproc empty {} {
    [self] instvar _tail _head
    if {$_head=="NIL" && $_tail=="NIL"} {
	return 1
    } else {
	return 0
    }
}
DoubleList instproc begin_iterator {name} {
    [self] instvar _head
    return [DLIterator $name -node $_head]
}
DoubleList instproc getelem_list {} {
    [self] instvar _head
    [self] begin_iterator [self]::it
    while {[[self]::it get]!="NIL"} {
	lappend ret [[self]::it next_elem]
    }
    [self]::it destroy
    return $ret
}
DoubleList instproc end_list {} {
    return NIL
}
# object ist ein Visitorobeject 
# jedes element der Liste wird aufgerufen 
# wird dem Visitor Object durch dei Methode bergeben 
# $object $methode $list_elem
DoubleList instproc foreach_elem {object method {classfilter *}} {
    [self] begin_iterator [self]::it
    while {[[self]::it get]!="NIL"} {
	if {$classfilter=="*"} {
	    $object $method [[self]::it next_elem]
	} else {
	    set elem [[self]::it next_elem]
	    if {[string match $classfilter [$elem info class]]} {
		$object $method [[self]::it next_elem]
	    }
	}
    }
    [self]::it destroy
}
# !! nocht nicht ansetztbar
DoubleList instproc foreach_elem_deep {object method {classfilter *}} {
    [self] begin_iterator [self]::it
    while {[[self]::it get]!="NIL"} {
	set act_elem [[self]::it next_elem]
	if {$classfilter=="*"} {
	    $object $method $act_elem
	} else {
	    set elem [[self]::it next_elem]
	    if {[string match $classfilter [$elem info class]]} {
		$object $method $act_elem
	    }
	}
	# Wenn actualles Alement hat DoubleList dann rekursiv aufrufen
	if {[$act_elem istype DoubleList]} {
	    $act_elem foreach_elem_deep $object $method $classfilter
	}
    }
    [self]::it destroy
}
# Kann benutzt werden um sicher zu stellen, dass alle Elemente
# eine Bedingung erfllen
DoubleList instproc conform {object method} {
    [self] begin_iterator [self]::it
    set test_res 1
    while {[[self]::it get]!="NIL"} {
	set act_elem [[self]::it next_elem]
	if {![set test_res [$object $method $act_elem]]} {
	    break
	}
    }
    [self]::it destroy
    return $test_res
}
# Smalltalk hat hier eine Spur hinterlassen
# Also sucht den erten der eine Bedingung erfllt
DoubleList instproc detect {object method} {
    [self] begin_iterator [self]::it
    set ergebnis {}
    while {[[self]::it get]!="NIL"} {
	set act_elem [[self]::it next_elem]
	if {[set test_res [$object $method $act_elem]]} {
	    set ergebnis $act_elem
	    break
	}
    }
    [self]::it destroy
    return $ergebnis
}
DoubleList instproc detect_deep {object method} {
    [self] begin_iterator [self]::it
    set ergebnis {}
    while {[[self]::it get]!="NIL"} {
	set act_elem [[self]::it next_elem]
	if {[set test_res [$object $method $act_elem]]} {
	    set ergebnis $act_elem
	    break
	}
	if {[$act_elem istype DoubleList]} {
	    if {[set ergebnis [$act_elem detect_deep $object $method]]!=""} {
		break
	    }
	}
    }
    [self]::it destroy
    return $ergebnis
}
