#####################################################################
# Copyright (C) 2000 Artur Trzewik
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
######################################################################
# Header
# CVS - $Id: dbitotcl_ext.tcl,v 1.1 2001/01/06 14:59:42 artur Exp $
#
# Information:
# Modul (benutzt von Schema Editor)
# Diese Klassen lesen das DB Schema mit Hilfe von Sql aus.
# Das Auslese-Proze geschiet mit Hilfe von Beusucher Muster.
# Besucher-Objekt soll die Schnittstelle besitzen:
# newtable {tablename}
# newattribute parlist->{{name ??} {not_null ?} {type ?} ...}
#
Class DBISchemaExtractor -superclass DBISql -parameter {
    {dbspecific {name not_null primary_key default type auto_increment}}
}
DBISchemaExtractor instproc extract_schema {visitor} {
    [self] instvar sqlhandle
    foreach table [$sqlhandle invoke tables] {
	$visitor newtable $table
	[self] extract_table $visitor $table
    }
}
DBISchemaExtractor instproc extract_table {visitor table} {
    [self] instvar sqlhandle
    set odbctypes {
	{1 char} {2 int} {3 decimal} {4 int} {5 smallint} {6 float}
	{7 double} {8 double} {9 date} {10 time} {11 timestamp} {12 varchar}
    }
    set kodbctypes [keylkeys odbctypes]
    set qhandle [$sqlhandle prepare "select * from $table"]
    $qhandle invoke execute
    set numfields [$qhandle attribute NUM_OF_FIELDS]
    set names [$qhandle attribute NAME]
    set precisions [$qhandle attribute PRECISION]
    set nullables [$qhandle attribute NULLABLE]
    set types [$qhandle attribute TYPE]                                                
    $qhandle finish

    for {set x 0} {$x<$numfields} {incr x} {
	set alist {}
	keylset alist name [lindex $names $x]
	if {[lindex $nullables $x]=="1"} {
	    keylset alist null 1
	}
	keylset alist length [lindex $precisions $x]
	set otype [lindex $types $x]
	if {[lcontain $kodbctypes $otype]} {
	    keylset alist type [keylget odbctypes $otype]
	} else {
	    # this is rather new dbi command probaby unsupported by
	    # old interfaces
	    if {![info exist typeinfo]} {
		foreach t [$sqlhandle invoke type_info] {
		    keylset typeinfo [keylget t DATA_TYPE] [keylget t TYPE_NAME] 
		}
	    }
	    # mysql produce "int unsigned" also only 1st elem 
	    keylset alist type [lindex [keylget typeinfo $otype] 0]
	}
	#puts "type [keylget alist type]"
	$visitor newattribute $alist
    }
}
