#####################################################################
# Copyright (C) 2000 Artur Trzewik
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
######################################################################
# Header
# CVS - $Id: formengine.tcl,v 1.11 2001/01/06 14:47:51 artur Exp $
#
# Information:
# Modul
# Implementierung von Check (Validating) Klassen
# benutzt beim berprfen von Benutzer-Eingaben
# Es beinhaltet auch die Konstruktor Methoden fr Instanzen aus XML tDOM

Class Validator -parameter {data} 
Validator instproc isvalid {value} {
    set ret [[self] isvalidPur $value]
    if {!$ret} {
	puts "Check Failed on Vaildator [[self] validInfo] by $value"
    }
    return $ret
}
Validator instproc validInfo {} {
    [self] instvar data
    return "[namespace tail [[self] info class]] data: $data"
}
Validator instproc asTDom {doc_elem} {
    [self] instvar data
    set check_node [$doc_elem createElement check]
    check_node setAttribute validator [namespace tail [[self] info class]]
    $check_node appendChild [$doc_elem createTextNode $data]
    return $check_node
}

# Konstruktor Methode aus tDOM
# z.B. <check validator="Regexp">^A</check>
Validator proc createFromTDom {name check_node} {
    return [Validator::[$check_node getAttribute validator] $name -data [$check_node text]]
}

Class Validator::Regexp -superclass Validator 
Validator::Regexp instproc isvalidPur {value} {
    [self] instvar data
    return [regexp $data $value]
}

Class Validator::Expr -superclass Validator 
Validator::Expr instproc isvalidPur {x} {
    [self] instvar data
    return [expr $data]
}

Class Validator::Length -superclass Validator 
Validator::Length instproc isvalidPur {value} {
    [self] instvar data
    return [expr [string length $value]<=$data]
}

#
# Eine geordnete Menge von Validatoren
#
Class ValidatorSet -superclass DoubleList
# Erzeugt eine ValidatorSet Instanz aus etwa
# <checks>
#  <check validator="Regexp">[0-9].</checks>
#  <check validator="Expr">$x<20</checks>
#  <check validator="Length">10</checks>
# </checks>
ValidatorSet proc createFromTDom {name checks_node} {
    ValidatorSet $name
    foreach check_node [$checks_node selectNodes child::check] {
	set vname [Validator createFromTDom \
		       ${name}::[Object autoname validator] $check_node]
	if {$vname==""} continue
	$name push_back $vname
    }
    return $name
}
ValidatorSet instproc isvalid {tvalue} {
    [self] instvar value
    set value $tvalue
    set res [[self] conform [self] validProc]
    unset value
    return $res
}
ValidatorSet instproc validProc {object} {
    [self] instvar value
    set ret [$object isvalid $value] 
    return $ret
}
ValidatorSet instproc asTDom {doc_elem} {
    [self] instvar checks_node mydoc_elem
    set mydoc_elem
    set checks_node [$doc_elem createElement checks]
    [self] foreach [self] addElem
    return $checks_node
}
ValidatorSet instproc addElem {object} {
    [self] instvar checks_node mydoc_elem
    $checks_node appendChild [$object asTDom mydoc_elem]
}
# 
# Erweiter den Interface von InputWidget um validate Prozeduren 
#
InputWidget instproc isvalid {} {
    [self] instvar value default
    [self] updatevalue
    if {[Object isobject [self]::validator]} {
	if {$value==$default} {return 1}
	return [[self]::validator isvalid $value]
    }
    return 1
}
WidgetContainer instproc isvalid {} {
    return [[self] conform [self] validProc]
}
WidgetContainer instproc validProc {object} {
    if {[[$object info class ] info instprocs isvalid]==""} {
	return [$object isvalid]
    }
    return 1
}
